#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# climalab/__init__.py

__version__ = "4.5.5"

# Define what should be available when using 'from climalab import *'
__all__ = [
    'meteorological',
    'netcdf_tools',
    'supplementary_tools'
]