#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# climalab/netcdf_tools/__init__.py

# Define what should be available when using 'from climalab.netcdf_tools import *'
__all__ = [
    'cdo_tools',
    'detect_faulty',
    'extract_basics',
    'nco_tools'
]